#ifndef MSP_GL_ORDEREDSCENE_H_
#define MSP_GL_ORDEREDSCENE_H_

#include <list>
#include "scene.h"

namespace Msp {
namespace GL {

/**
A scene that renders its contents in a specific order.  Inserting Renderables
in the middle and removing them are O(N) operations.
*/
class OrderedScene: public Scene
{
private:
	typedef std::list<const Renderable *> RenderableList;

	RenderableList renderables;

public:
	virtual void add(const Renderable &);
	virtual void remove(const Renderable &);
	void prepend(const Renderable &);
	void insert(unsigned, const Renderable &);
	void insert_after(const Renderable &, const Renderable &);

	virtual void setup_frame() const;
	virtual void finish_frame() const;

	using Scene::render;
	virtual void render(Renderer &, const Tag & = Tag()) const;
};

} // namespace GL
} // namespace Msp

#endif
